/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2014 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE PACKAGE BODY xxas_rslt_cache_pkg AS 

--
-- Private Global Types and Constants
--
g_PACKAGE_NAME CONSTANT varchar2(30) := 'xxas_rslt_cache_pkg';

TYPE g_emp_menu_r IS RECORD
       (function_name     fnd_form_functions.function_name%TYPE
       ,menu_has_emp_flag boolean);
       
TYPE g_emp_menu_t IS TABLE OF g_emp_menu_r INDEX BY binary_integer;

g_emp_menu_tbl          g_emp_menu_t;

-- ------------------------------------------------------------------------ *
-- Name    : search_facet_table
-- ------------------------------------------------------------------------ *
FUNCTION search_facet_table
          (p_userenv_lang IN VARCHAR2
          ,p_business_group_id  IN NUMBER DEFAULT NULL)
RETURN g_src_criteria_t
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (pay_user_tables,pay_user_columns,pay_user_rows_f,pay_user_column_instances_f)
  $END

IS
	CURSOR csr_facet_list IS
  SELECT put.business_group_id
        ,xxas_util_pkg.convert_to_camelCase(pur.row_low_range_or_name) criteria
        ,purv.row_low_range_or_name friendly_name
        ,searchv.value searchable
        ,decode(mrv.value,'Y','select-multiple','select-multiple') facet_type
        ,5 max_size
        ,decode(mrv.value,'Y','alpha',null) order_by
        ,pur.display_sequence
  FROM   pay_user_tables put
        ,pay_user_columns perm
        ,pay_user_columns search
        ,pay_user_columns facet
        ,pay_user_columns multirow
        ,pay_user_rows_f pur
        ,pay_user_rows_f_vl purv
        ,pay_user_column_instances_f permv
        ,pay_user_column_instances_f searchv
        ,pay_user_column_instances_f facetv
        ,pay_user_column_instances_f mrv
  WHERE  put.user_table_name = 'XXAS_SRC_CRITERIA'
  AND    put.business_group_id IS NOT NULL
  AND    put.legislation_code IS NULL
  AND    put.user_table_id = perm.user_table_id
  AND    put.user_table_id = search.user_table_id
  AND    put.user_table_id = facet.user_table_id
  AND    put.user_table_id = multirow.user_table_id
  AND    perm.business_group_id IS NOT NULL
  AND    perm.legislation_code IS NULL
  AND    perm.user_column_name = 'Permission'
  AND    search.business_group_id IS NOT NULL
  AND    search.legislation_code IS NULL
  AND    search.user_column_name = 'Searchable'
  AND    facet.business_group_id IS NOT NULL
  AND    facet.legislation_code IS NULL
  AND    facet.user_column_name = 'Use as Facet'
  AND    multirow.business_group_id IS NOT NULL
  AND    multirow.legislation_code IS NULL
  AND    multirow.user_column_name = 'Multirow'
  AND    put.user_table_id = pur.user_table_id
  AND    pur.business_group_id IS NOT NULL
  AND    pur.legislation_code IS NULL
  AND    perm.user_column_id = permv.user_column_id
  AND    pur.user_row_id = permv.user_row_id
  AND    permv.business_group_id IS NOT NULL
  AND    permv.legislation_code IS NULL
  AND    search.user_column_id = searchv.user_column_id
  AND    pur.user_row_id = searchv.user_row_id
  AND    searchv.business_group_id IS NOT NULL
  AND    searchv.legislation_code IS NULL
  AND    facet.user_column_id = facetv.user_column_id
  AND    pur.user_row_id = facetv.user_row_id
  AND    facetv.business_group_id IS NOT NULL
  AND    facetv.legislation_code IS NULL
  AND    multirow.user_column_id = mrv.user_column_id
  AND    pur.user_row_id = mrv.user_row_id
  AND    mrv.business_group_id IS NOT NULL
  AND    mrv.legislation_code IS NULL
  AND    trunc(sysdate) BETWEEN
         pur.effective_start_date AND pur.effective_end_date
  AND    trunc(sysdate) BETWEEN
         permv.effective_start_date AND permv.effective_end_date
  AND    trunc(sysdate) BETWEEN
         searchv.effective_start_date AND searchv.effective_end_date
  AND    trunc(sysdate) BETWEEN
         facetv.effective_start_date AND facetv.effective_end_date
  AND    trunc(sysdate) BETWEEN
         mrv.effective_start_date AND mrv.effective_end_date
  AND    facetv.value = 'Y'     
  AND    purv.user_row_id = pur.user_row_id  
  ORDER BY pur.display_sequence, pur.row_low_range_or_name;
	
  l_src_criteria_t	g_src_criteria_t;
  
  facet_rec 				csr_facet_list%ROWTYPE;
	
  facet_index				NUMBER := 1;	
  l_facet_exists    boolean := FALSE;

BEGIN

  FOR facet_rec IN csr_facet_list LOOP
    
    IF facet_index > 1 THEN
      FOR i IN 1..facet_index - 1 LOOP
        IF l_src_criteria_t(i).criteria = facet_rec.criteria THEN
          l_facet_exists := TRUE;          
        END IF;
      END LOOP;
    END IF;
  
    IF l_facet_exists = FALSE THEN
      l_src_criteria_t(facet_index).business_group_id := facet_rec.business_group_id;
      l_src_criteria_t(facet_index).facet_number  := facet_index;
      l_src_criteria_t(facet_index).facet_type  := facet_rec.facet_type;
			l_src_criteria_t(facet_index).attribute_name:= facet_rec.criteria;
      l_src_criteria_t(facet_index).criteria := facet_rec.criteria;
      l_src_criteria_t(facet_index).friendly_name := facet_rec.friendly_name;
      l_src_criteria_t(facet_index).max_size := facet_rec.max_size;
      l_src_criteria_t(facet_index).order_by := facet_rec.order_by;
      l_src_criteria_t(facet_index).searchable := facet_rec.searchable;
      l_src_criteria_t(facet_index).display_sequence := facet_rec.display_sequence;
      facet_index := facet_index + 1;
    END IF;

    l_facet_exists := FALSE; 
  
  END LOOP;
  	
	RETURN l_src_criteria_t;
	
END search_facet_table;

-- ------------------------------------------------------------------------ *
-- Name    : encode_lookup
-- ------------------------------------------------------------------------ *
FUNCTION encode_lookup
  (p_lookup_type    IN varchar2
  ,p_meaning        IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_lookup_values)
  $END
IS

  l_return fnd_lookup_values.lookup_code%TYPE;

  --
  -- Gets the lookup code using the meaning
  --
  CURSOR csr_lookup IS
  SELECT hrl.lookup_code
  FROM   hr_lookups hrl
  WHERE  hrl.lookup_type = p_lookup_type
  AND    hrl.meaning = p_meaning;

BEGIN

  --
  -- Check for null parameters
  --
  IF p_lookup_type IS NOT NULL AND
     p_meaning     IS NOT NULL THEN
    --
    -- Encode the lookup
    --
    OPEN  csr_lookup;
    FETCH csr_lookup INTO l_return;
    CLOSE csr_lookup;

  END IF;

  RETURN l_return;

END encode_lookup;

-- ------------------------------------------------------------------------ *
-- Name    : menu_name_to_id
-- ------------------------------------------------------------------------ *
FUNCTION menu_name_to_id
  (p_menu_name IN varchar2) RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_menus)
  $END
IS

  l_menu_id number;

  CURSOR csr_name_to_id IS
  SELECT m.menu_id
  FROM   fnd_menus m
  WHERE  m.menu_name = p_menu_name;
  
BEGIN

  IF p_menu_name IS NOT NULL THEN
    OPEN   csr_name_to_id;
    FETCH  csr_name_to_id INTO l_menu_id;
    CLOSE  csr_name_to_id;
  END IF;

  RETURN l_menu_id;

END menu_name_to_id;

-- ------------------------------------------------------------------------ *
-- Name    : menu_id_to_name
-- ------------------------------------------------------------------------ *
FUNCTION menu_id_to_name
  (p_menu_id IN number) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_menus)
  $END
IS

  l_menu_name fnd_menus.menu_name%TYPE;

  CURSOR csr_id_to_name IS
  SELECT m.menu_name
  FROM   fnd_menus m
  WHERE  m.menu_id = p_menu_id;
  
BEGIN

  IF p_menu_id IS NOT NULL THEN
    OPEN   csr_id_to_name;
    FETCH  csr_id_to_name INTO l_menu_name;
    CLOSE  csr_id_to_name;
  END IF;

  RETURN l_menu_name;

END menu_id_to_name;

-- ------------------------------------------------------------------------ *
-- Name    : get_menu_entries
-- ------------------------------------------------------------------------ *
FUNCTION get_menu_entries
  (p_menu_id                IN number
  ,p_userenv_lang           IN varchar2) RETURN g_menu_entry_t
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_menu_entries, fnd_menu_entries_tl, fnd_form_functions, fnd_objects)
  $END
IS

  l_PROC CONSTANT  varchar2(30) := 'get_menu_entries';
  l_menu_entry_tbl g_menu_entry_t;
  i                number := 1;
  
  CURSOR csr_menu_entries IS
  SELECT   e.menu_id parent_menu_id
          ,e.entry_sequence
          ,e.prompt
          ,e.description
          ,e.sub_menu_id menu_id
          ,cm.menu_name menu_name
          ,e.function_id
          ,cf.function_name
          ,cf.object_id
          ,cf.obj_name
          ,cf.parameters function_parameters
          ,cf.web_icon function_icon
  FROM     fnd_menu_entries_vl e
          ,fnd_menus_vl cm
          ,(SELECT f.function_id
                  ,f.function_name
                  ,f.object_id
                  ,o.obj_name
                  ,f.parameters
                  ,f.web_icon
            FROM   fnd_form_functions_vl f
                  ,fnd_objects o
            WHERE  f.object_id = o.object_id(+)) cf
  WHERE    e.function_id = cf.function_id(+)
  AND      e.sub_menu_id = cm.menu_id(+)
  AND      e.menu_id = p_menu_id
  ORDER BY e.entry_sequence;

BEGIN

  IF p_menu_id IS NULL THEN
    RETURN l_menu_entry_tbl;
  END IF;
  
  FOR csr_rec IN csr_menu_entries LOOP
    l_menu_entry_tbl(i).parent_menu_id      := csr_rec.parent_menu_id;
    l_menu_entry_tbl(i).entry_sequence      := csr_rec.entry_sequence;    
    l_menu_entry_tbl(i).prompt              := csr_rec.prompt;
    l_menu_entry_tbl(i).description         := csr_rec.description;
    l_menu_entry_tbl(i).menu_id             := csr_rec.menu_id;
    l_menu_entry_tbl(i).menu_name           := csr_rec.menu_name;
    l_menu_entry_tbl(i).function_id         := csr_rec.function_id;
    l_menu_entry_tbl(i).function_name       := csr_rec.function_name;
    l_menu_entry_tbl(i).object_id           := csr_rec.object_id;
    l_menu_entry_tbl(i).obj_name            := csr_rec.obj_name;
    l_menu_entry_tbl(i).function_parameters := csr_rec.function_parameters;
    l_menu_entry_tbl(i).function_icon       := csr_rec.function_icon;
    i := i + 1;
  END LOOP;
  
  RETURN l_menu_entry_tbl;

END get_menu_entries;

-- ------------------------------------------------------------------------ *
-- Name    : get_resp_id_from_key
-- ------------------------------------------------------------------------ *
FUNCTION get_resp_id_from_key
  (p_responsibility_key IN varchar2
  ,p_application_id     IN number) RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON(fnd_responsibility)
  $END
IS

  l_PROC CONSTANT varchar2(30) := 'get_resp_id_from_key';
  l_resp_id       number;
  
  CURSOR csr_resp_id IS
  SELECT resp.responsibility_id
  FROM   fnd_responsibility resp
  WHERE  resp.responsibility_key = p_responsibility_key
  AND    resp.application_id = p_application_id;

BEGIN

  IF p_responsibility_key IS NULL OR
     p_application_id     IS NULL THEN
    RETURN null;
  END IF;
  
  OPEN  csr_resp_id;
  FETCH csr_resp_id INTO l_resp_id;
  CLOSE csr_resp_id;
  
  RETURN l_resp_id;

END get_resp_id_from_key;

-- ------------------------------------------------------------------------ *
-- Name    : get_resp_functions
-- ------------------------------------------------------------------------ *
FUNCTION get_resp_functions
  (p_responsibility_id   IN number
  ,p_resp_application_id IN number
  ,p_resp_menu_id        IN number
  ,p_language_code       IN varchar2
  ,p_inc_funcs_with_no_prompt IN varchar2 default 'N') RETURN g_resp_function_t
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON
      (fnd_menu_entries
      ,fnd_menu_entries_tl
      ,fnd_form_functions
      ,fnd_menus)
  $END
IS

  l_PROC CONSTANT  varchar2(30) := 'get_resp_functions';
  l_resp_func_tbl  g_resp_function_t;
  i                number := 1;
  
  CURSOR csr_resp_functions IS
  SELECT   me.function_id
          ,f.type function_type
          ,me.prompt
          ,me.description
          ,substr(substr(SYS_CONNECT_BY_PATH(me.prompt, ' -> '), 5)
                 ,1
                 ,length(substr(SYS_CONNECT_BY_PATH(me.prompt, ' -> '), 5))
                   - length(me.prompt) - 4) path
  FROM     fnd_menu_entries_vl me
          ,fnd_form_functions f
          ,fnd_menus m
  WHERE    me.function_id = f.function_id(+)
  AND      me.menu_id = m.menu_id(+)
  AND      f.type IN ('FORM', 'JSP')
--  AND      me.prompt IS NOT NULL
   AND  ( (p_inc_funcs_with_no_prompt = 'N' and  me.prompt IS NOT NULL)
    or (p_inc_funcs_with_no_prompt = 'Y') )
  AND      me.grant_flag = 'Y'
  AND      me.function_id NOT IN
             (SELECT frf.action_id 
              FROM   fnd_resp_functions frf
              WHERE  frf.responsibility_id = p_responsibility_id
              AND    frf.application_id = p_resp_application_id
              AND    frf.rule_type = 'F')
  AND      me.menu_id NOT IN
             (SELECT frf.action_id 
              FROM   fnd_resp_functions frf
              WHERE  frf.responsibility_id = p_responsibility_id
              AND    frf.application_id = p_resp_application_id
              AND    frf.rule_type = 'M')
  CONNECT BY PRIOR me.sub_menu_id = me.menu_id 
  AND PRIOR me.prompt IS NOT NULL
  START WITH me.menu_id = p_resp_menu_id
  ORDER SIBLINGS BY
           nvl2(me.function_id, 1, 2)
          ,me.entry_sequence;

BEGIN

  IF p_responsibility_id   IS NULL OR
     p_resp_application_id IS NULL OR
     p_resp_menu_id        IS NULL OR
     p_language_code       IS NULL THEN
    RETURN l_resp_func_tbl;
  END IF;
  
  FOR csr_rec IN csr_resp_functions LOOP
    l_resp_func_tbl(i).function_id   := csr_rec.function_id;
    l_resp_func_tbl(i).function_type := csr_rec.function_type;
    l_resp_func_tbl(i).prompt        := csr_rec.prompt;    
    l_resp_func_tbl(i).description   := csr_rec.description;
    l_resp_func_tbl(i).path          := csr_rec.path;
    i := i + 1;
  END LOOP;
  
  RETURN l_resp_func_tbl;

END get_resp_functions;

-- ------------------------------------------------------------------------ *
-- Name    : object_name
-- ------------------------------------------------------------------------ *
FUNCTION object_name
  (p_object_id IN number) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_objects)
  $END
IS

  l_obj_name fnd_objects.obj_name%TYPE;

BEGIN

  IF p_object_id IS NOT NULL THEN
    SELECT obj_name
    INTO   l_obj_name
    FROM   fnd_objects
    WHERE  object_id = p_object_id;
  END IF;
  
  RETURN l_obj_name;
 
EXCEPTION

  WHEN no_data_found THEN
    RETURN l_obj_name;

END object_name;

-- ------------------------------------------------------------------------ *
-- Name    : object_id
-- ------------------------------------------------------------------------ *
FUNCTION object_id
  (p_object_name IN varchar2) RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_objects)
  $END
IS

  l_object_id number;

BEGIN

  IF p_object_name IS NOT NULL THEN
    SELECT object_id
    INTO   l_object_id
    FROM   fnd_objects
    WHERE  obj_name = p_object_name;
  END IF;
  
  RETURN l_object_id;
 
EXCEPTION

  WHEN no_data_found THEN
    RETURN l_object_id;

END object_id;

-- ------------------------------------------------------------------------ *
-- Name    : object_db_name
-- ------------------------------------------------------------------------ *
FUNCTION object_db_name
  (p_object_name IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_objects)
  $END
IS

  l_db_obj_name fnd_objects.database_object_name%TYPE;

BEGIN

  IF p_object_name IS NOT NULL THEN
    SELECT database_object_name
    INTO   l_db_obj_name
    FROM   fnd_objects
    WHERE  obj_name = p_object_name;
  END IF;
  
  RETURN l_db_obj_name;
 
EXCEPTION

  WHEN no_data_found THEN
    RETURN l_db_obj_name;

END object_db_name;

-- ------------------------------------------------------------------------ *
-- Name    : object_pk1_column_name
-- ------------------------------------------------------------------------ *
FUNCTION object_pk1_column_name
  (p_object_name IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_objects)
  $END
IS

  l_col_name fnd_objects.pk1_column_name%TYPE;

BEGIN

  IF p_object_name IS NOT NULL THEN
    SELECT pk1_column_name
    INTO   l_col_name
    FROM   fnd_objects
    WHERE  obj_name = p_object_name;
  END IF;
  
  RETURN l_col_name;
 
EXCEPTION

  WHEN no_data_found THEN
    RETURN l_col_name;

END object_pk1_column_name;

-- ------------------------------------------------------------------------ *
-- Name    : object_pk2_column_name
-- ------------------------------------------------------------------------ *
FUNCTION object_pk2_column_name
  (p_object_name IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_objects)
  $END
IS

  l_col_name fnd_objects.pk2_column_name%TYPE;

BEGIN

  IF p_object_name IS NOT NULL THEN
    SELECT pk2_column_name
    INTO   l_col_name
    FROM   fnd_objects
    WHERE  obj_name = p_object_name;
  END IF;
  
  RETURN l_col_name;
 
EXCEPTION

  WHEN no_data_found THEN
    RETURN l_col_name;

END object_pk2_column_name;

-- ------------------------------------------------------------------------ *
-- Name    : object_pk3_column_name
-- ------------------------------------------------------------------------ *
FUNCTION object_pk3_column_name
  (p_object_name IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_objects)
  $END
IS

  l_col_name fnd_objects.pk3_column_name%TYPE;

BEGIN

  IF p_object_name IS NOT NULL THEN
    SELECT pk3_column_name
    INTO   l_col_name
    FROM   fnd_objects
    WHERE  obj_name = p_object_name;
  END IF;
  
  RETURN l_col_name;
 
EXCEPTION

  WHEN no_data_found THEN
    RETURN l_col_name;

END object_pk3_column_name;

-- ------------------------------------------------------------------------ *
-- Name    : object_pk4_column_name
-- ------------------------------------------------------------------------ *
FUNCTION object_pk4_column_name
  (p_object_name IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_objects)
  $END
IS

  l_col_name fnd_objects.pk4_column_name%TYPE;

BEGIN

  IF p_object_name IS NOT NULL THEN
    SELECT pk4_column_name
    INTO   l_col_name
    FROM   fnd_objects
    WHERE  obj_name = p_object_name;
  END IF;
  
  RETURN l_col_name;
 
EXCEPTION

  WHEN no_data_found THEN
    RETURN l_col_name;

END object_pk4_column_name;

-- ------------------------------------------------------------------------ *
-- Name    : object_pk5_column_name
-- ------------------------------------------------------------------------ *
FUNCTION object_pk5_column_name
  (p_object_name IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_objects)
  $END
IS

  l_col_name fnd_objects.pk5_column_name%TYPE;

BEGIN

  IF p_object_name IS NOT NULL THEN
    SELECT pk5_column_name
    INTO   l_col_name
    FROM   fnd_objects
    WHERE  obj_name = p_object_name;
  END IF;
  
  RETURN l_col_name;
 
EXCEPTION

  WHEN no_data_found THEN
    RETURN l_col_name;

END object_pk5_column_name;

-- ------------------------------------------------------------------------ *
-- Name    : object_pk1_column_type
-- ------------------------------------------------------------------------ *
FUNCTION object_pk1_column_type
  (p_object_name IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_objects)
  $END
IS

  l_col_type fnd_objects.pk1_column_type%TYPE;

BEGIN

  IF p_object_name IS NOT NULL THEN
    SELECT pk1_column_type
    INTO   l_col_type
    FROM   fnd_objects
    WHERE  obj_name = p_object_name;
  END IF;
  
  RETURN l_col_type;
 
EXCEPTION

  WHEN no_data_found THEN
    RETURN l_col_type;

END object_pk1_column_type;

-- ------------------------------------------------------------------------ *
-- Name    : object_pk2_column_type
-- ------------------------------------------------------------------------ *
FUNCTION object_pk2_column_type
  (p_object_name IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_objects)
  $END
IS

  l_col_type fnd_objects.pk2_column_type%TYPE;

BEGIN

  IF p_object_name IS NOT NULL THEN
    SELECT pk2_column_type
    INTO   l_col_type
    FROM   fnd_objects
    WHERE  obj_name = p_object_name;
  END IF;
  
  RETURN l_col_type;
 
EXCEPTION

  WHEN no_data_found THEN
    RETURN l_col_type;

END object_pk2_column_type;

-- ------------------------------------------------------------------------ *
-- Name    : object_pk3_column_type
-- ------------------------------------------------------------------------ *
FUNCTION object_pk3_column_type
  (p_object_name IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_objects)
  $END
IS

  l_col_type fnd_objects.pk3_column_type%TYPE;

BEGIN

  IF p_object_name IS NOT NULL THEN
    SELECT pk3_column_type
    INTO   l_col_type
    FROM   fnd_objects
    WHERE  obj_name = p_object_name;
  END IF;
  
  RETURN l_col_type;
 
EXCEPTION

  WHEN no_data_found THEN
    RETURN l_col_type;

END object_pk3_column_type;

-- ------------------------------------------------------------------------ *
-- Name    : object_pk4_column_type
-- ------------------------------------------------------------------------ *
FUNCTION object_pk4_column_type
  (p_object_name IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_objects)
  $END
IS

  l_col_type fnd_objects.pk4_column_type%TYPE;

BEGIN

  IF p_object_name IS NOT NULL THEN
    SELECT pk4_column_type
    INTO   l_col_type
    FROM   fnd_objects
    WHERE  obj_name = p_object_name;
  END IF;
  
  RETURN l_col_type;
 
EXCEPTION

  WHEN no_data_found THEN
    RETURN l_col_type;

END object_pk4_column_type;

-- ------------------------------------------------------------------------ *
-- Name    : object_pk5_column_type
-- ------------------------------------------------------------------------ *
FUNCTION object_pk5_column_type
  (p_object_name IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_objects)
  $END
IS

  l_col_type fnd_objects.pk5_column_type%TYPE;

BEGIN

  IF p_object_name IS NOT NULL THEN
    SELECT pk5_column_type
    INTO   l_col_type
    FROM   fnd_objects
    WHERE  obj_name = p_object_name;
  END IF;
  
  RETURN l_col_type;
 
EXCEPTION

  WHEN no_data_found THEN
    RETURN l_col_type;

END object_pk5_column_type;

-- ------------------------------------------------------------------------ *
-- Name    : get_emp_functions
-- ------------------------------------------------------------------------ *
FUNCTION get_emp_functions RETURN xxas_com_sec_pkg.g_number_t
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_form_functions)
  $END
IS

  l_func_tbl xxas_com_sec_pkg.g_number_t;

  CURSOR csr_emp_functions IS
  SELECT f.function_id
  FROM   fnd_form_functions f
  WHERE  f.web_html_call in( 'OA.jsp?page=/xxas/oracle/apps/xxas/emp/webui/EmployeeCenterPG','xxas/XxasEmp.jsp')
  ORDER BY decode(f.function_name, 'XXAS_EMP_HTML_SS',1,'XXAS_EMP_SS', 3, 2);
  
BEGIN

  OPEN   csr_emp_functions;
  FETCH  csr_emp_functions BULK COLLECT INTO l_func_tbl;
  CLOSE  csr_emp_functions;

  RETURN l_func_tbl;

END get_emp_functions;


-- ------------------------------------------------------------------------ *
-- Name    : get_emp_menu_function
-- ------------------------------------------------------------------------ *
FUNCTION get_emp_menu_function
  (p_menu_id IN number) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_menu_entries)
  $END
IS

  l_func_tbl      xxas_com_sec_pkg.g_number_t;
  l_function_name fnd_form_functions.function_name%TYPE;
  l_function_id   fnd_form_functions.function_id%TYPE; 

  CURSOR csr_menu_walk
    (cp_function_id IN number)
  IS
  SELECT e.function_id
  FROM   fnd_menu_entries e
  WHERE  e.function_id = cp_function_id
  AND    e.grant_flag = 'Y'
  AND    e.menu_id NOT IN
           (SELECT g.menu_id
            FROM   fnd_menus g
            WHERE  g.menu_name = 'ICX_STANDARD_GLOBAL_MENU')
  CONNECT BY PRIOR e.sub_menu_id = e.menu_id
  AND    e.function_id = cp_function_id
  AND    e.grant_flag = 'Y'
  AND    e.menu_id NOT IN
           (SELECT g.menu_id
            FROM   fnd_menus g
            WHERE  g.menu_name = 'ICX_STANDARD_GLOBAL_MENU')
  START WITH e.menu_id = p_menu_id;

BEGIN

  IF p_menu_id IS NULL THEN
    RETURN NULL;
  END IF;
  
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    IF g_emp_menu_tbl.exists(p_menu_id) THEN
      IF g_emp_menu_tbl(p_menu_id).menu_has_emp_flag THEN
        RETURN g_emp_menu_tbl(p_menu_id).function_name;
      ELSE
        RETURN NULL;
      END IF;
    END IF;    
  $END

  l_func_tbl := get_emp_functions;
  
  IF l_func_tbl.count = 0 THEN
    RETURN NULL;
  ELSE
    FOR i IN l_func_tbl.first..l_func_tbl.last LOOP

      OPEN  csr_menu_walk(l_func_tbl(i));
      FETCH csr_menu_walk INTO l_function_id;
      CLOSE csr_menu_walk;

      IF l_function_id IS NOT NULL THEN
        l_function_name := get_function_name(l_func_tbl(i));

        $IF DBMS_DB_VERSION.VER_LE_10 $THEN
          g_emp_menu_tbl(p_menu_id).menu_has_emp_flag := TRUE;
          g_emp_menu_tbl(p_menu_id).function_name     := l_function_name;        
        $END
        RETURN l_function_name;      
      END IF;
    END LOOP;
  END IF;

  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    g_emp_menu_tbl(p_menu_id).menu_has_emp_flag := FALSE;
  $END
  RETURN NULL; 

END get_emp_menu_function;

-- ------------------------------------------------------------------------ *
-- Name    : menu_has_emp_flag
-- ------------------------------------------------------------------------ *
FUNCTION menu_has_emp_flag
  (p_menu_id IN number) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    ;
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_menu_entries)
  $END
IS

BEGIN

  IF get_emp_menu_function(p_menu_id) IS NULL THEN
    RETURN 'N';
  ELSE
    RETURN 'Y';
  END IF;

END menu_has_emp_flag;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_product_status
-- ------------------------------------------------------------------------ *
FUNCTION get_product_status
  (p_prod_key IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (xxas_com_products)
  $END
IS

  CURSOR csr_info
  IS
  SELECT p.status
  FROM   xxas_com_products p
  WHERE  p.prod_key = p_prod_key;
  
  l_status xxas_com_products.status%TYPE;

BEGIN

  OPEN  csr_info;
  FETCH csr_info INTO l_status;
  CLOSE csr_info;

  RETURN l_status;

END get_product_status;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_product_signature
-- ------------------------------------------------------------------------ *
FUNCTION get_product_signature
  (p_prod_key IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (xxas_com_products)
  $END
IS

  CURSOR csr_info
  IS
  SELECT p.signature
  FROM   xxas_com_products p
  WHERE  p.prod_key = p_prod_key;
  
  l_signature xxas_com_products.signature%TYPE;

BEGIN

  OPEN  csr_info;
  FETCH csr_info INTO l_signature;
  CLOSE csr_info;

  RETURN l_signature;

END get_product_signature;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_function_id
-- ------------------------------------------------------------------------ *
FUNCTION get_function_id
  (p_function_name IN VARCHAR2) RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_form_functions)
  $END
IS

  CURSOR csr_func_id
  IS                
  SELECT func.function_id
  FROM   fnd_form_functions func
  WHERE  func.function_name  = p_function_name;
  
  l_function_id number;

BEGIN
  
  IF p_function_name IS NOT NULL THEN
    OPEN  csr_func_id;
    FETCH csr_func_id INTO l_function_id;
    CLOSE csr_func_id;
  END IF;
  
  RETURN l_function_id;

END get_function_id;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_function_name
-- ------------------------------------------------------------------------ *
FUNCTION get_function_name
  (p_function_id IN NUMBER) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_form_functions)
  $END
IS

  CURSOR csr_func_name
  IS                
  SELECT func.function_name
  FROM   fnd_form_functions func
  WHERE  func.function_id  = p_function_id;
  
  l_function_name fnd_form_functions.function_name%TYPE;

BEGIN
  
  IF p_function_id IS NOT NULL THEN
    OPEN  csr_func_name;
    FETCH csr_func_name INTO l_function_name;
    CLOSE csr_func_name;
  END IF;
  
  RETURN l_function_name;

END get_function_name;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_object_name_for_func
-- ------------------------------------------------------------------------ *
  FUNCTION get_object_name_for_func
    (p_function_id IN VARCHAR2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_form_functions)
  $END
IS

  CURSOR csr_obj_name IS
  SELECT obj.obj_name
  FROM   fnd_objects obj
        ,fnd_form_functions ff
  WHERE  ff.function_id = p_function_id
  AND    ff.object_id = obj.object_id;
  
  l_obj_name fnd_objects.obj_name%TYPE;

BEGIN
  
  IF p_function_id IS NOT NULL THEN
    OPEN  csr_obj_name;
    FETCH csr_obj_name INTO l_obj_name;
    CLOSE csr_obj_name;
  END IF;
  
  RETURN l_obj_name;

END get_object_name_for_func;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_instance_set_id
-- ------------------------------------------------------------------------ *
FUNCTION get_instance_set_id
  (p_instance_set_name IN varchar2) RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_object_instance_sets)
  $END
IS

  CURSOR csr_id
  IS                
  SELECT fois.instance_set_id
  FROM   fnd_object_instance_sets fois
  WHERE  fois.instance_set_name = p_instance_set_name;
  
  l_id number;

BEGIN
  
  IF p_instance_set_name IS NOT NULL THEN
    OPEN  csr_id;
    FETCH csr_id INTO l_id;
    CLOSE csr_id;
  END IF;
  
  RETURN l_id;

END get_instance_set_id;

--
-- ------------------------------------------------------------------------ *
-- Name    : product_version
-- ------------------------------------------------------------------------ *
  FUNCTION product_version
    (p_prod_key IN VARCHAR2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (xxas_com_products)
  $END
IS

  CURSOR csr_prod_ver IS
  SELECT p.version
  FROM   xxas_com_products p
  WHERE  p.prod_key = p_prod_key;

  l_version xxas_com_products.version%TYPE;
  
BEGIN
  
  IF p_prod_key IS NOT NULL THEN
    OPEN  csr_prod_ver;
    FETCH csr_prod_ver INTO l_version;
    CLOSE csr_prod_ver;
  END IF;
  
  RETURN l_version;

END product_version;

-- ------------------------------------------------------------------------ *
-- Name    : get_ff_type_id
-- ------------------------------------------------------------------------ *
FUNCTION get_ff_type_id
  (p_formula_type_name IN varchar2) RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (ff_formula_types)
  $END
IS

  l_formula_type_id NUMBER;
  
BEGIN

  IF p_formula_type_name IS NULL THEN
    RETURN NULL;
  ELSE
    SELECT ft.formula_type_id
    INTO   l_formula_type_id
    FROM   ff_formula_types ft
    WHERE  ft.formula_type_name = p_formula_type_name;
  END IF;

  RETURN l_formula_type_id;
  
EXCEPTION

  WHEN no_data_found THEN
    RETURN null;

END get_ff_type_id;

-- ------------------------------------------------------------------------ *
-- Name    : get_ff_type_id_from_ff
-- ------------------------------------------------------------------------ *
FUNCTION get_ff_type_id_from_ff
  (p_formula_id     IN number
  ,p_effective_date IN date) RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (ff_formula_types)
  $END
IS

  l_formula_type_id NUMBER;
  
BEGIN

  IF p_formula_id IS NULL THEN
    RETURN NULL;
  ELSE
    SELECT ff.formula_type_id
    INTO   l_formula_type_id
    FROM   ff_formulas_f ff
    WHERE  ff.formula_id = p_formula_id
    AND    trunc(nvl(p_effective_date, sysdate)) BETWEEN
           ff.effective_start_date AND ff.effective_end_date;
  END IF;

  RETURN l_formula_type_id;
  
EXCEPTION

  WHEN no_data_found THEN
    RETURN null;

END get_ff_type_id_from_ff;

-- ------------------------------------------------------------------------ *
-- Name    : get_ff_type_name
-- ------------------------------------------------------------------------ *
FUNCTION get_ff_type_name
  (p_formula_type_id IN number) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (ff_formula_types)
  $END
IS

  l_formula_type_name ff_formula_types.formula_type_name%TYPE;
  
BEGIN

  IF p_formula_type_id IS NULL THEN
    RETURN NULL;
  ELSE
    SELECT ft.formula_type_name
    INTO   l_formula_type_name
    FROM   ff_formula_types ft
    WHERE  ft.formula_type_id = p_formula_type_id;
  END IF;

  RETURN l_formula_type_name;
  
EXCEPTION

  WHEN no_data_found THEN
    RETURN null;

END get_ff_type_name;

-- ------------------------------------------------------------------------ *
-- Name    : get_ff_id
-- ------------------------------------------------------------------------ *
FUNCTION get_ff_id
  (p_formula_name      IN varchar2
  ,p_formula_type_id   IN number
  ,p_effective_date    IN date
  ,p_business_group_id IN number   DEFAULT null
  ,p_legislation_code  IN varchar2 DEFAULT null) RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (ff_formulas_f)
  $END
IS

  l_formula_id NUMBER;
  
  CURSOR csr_ff_id IS
  SELECT ff.formula_id
  FROM   ff_formulas_f ff
  WHERE  ff.formula_name = p_formula_name
  AND    ff.formula_type_id = nvl(p_formula_type_id, ff.formula_type_id)
  AND    p_effective_date BETWEEN
         ff.effective_start_date AND ff.effective_end_date
  AND    nvl(ff.business_group_id, -1) = nvl(p_business_group_id, -1)
  AND    nvl(ff.legislation_code, xxas_util_pkg.hr_varchar2) =
         nvl(p_legislation_code, xxas_util_pkg.hr_varchar2);
  
BEGIN

  IF p_formula_name    IS NULL OR
     p_effective_date  IS NULL THEN
    RETURN NULL;
  ELSE
    OPEN  csr_ff_id;
    FETCH csr_ff_id INTO l_formula_id;
    CLOSE csr_ff_id;
  END IF;

  RETURN l_formula_id;

END get_ff_id;

-- ------------------------------------------------------------------------ *
-- Name    : get_vset_id_by_name
-- ------------------------------------------------------------------------ *
FUNCTION get_vset_id_by_name
  (p_value_set_name IN varchar2) RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_flex_value_sets)
  $END
IS

  l_value_set_id  number;
  
  CURSOR csr_vset_id IS
  SELECT vs.flex_value_set_id
  FROM   fnd_flex_value_sets vs
  WHERE  vs.flex_value_set_name = p_value_set_name;
  
BEGIN

  IF p_value_set_name IS NOT NULL THEN
    OPEN  csr_vset_id;
    FETCH csr_vset_id INTO l_value_set_id;
    CLOSE csr_vset_id;
  END IF;

  RETURN l_value_set_id;

END get_vset_id_by_name;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_kff_segment_name
-- ------------------------------------------------------------------------ *
FUNCTION get_kff_segment_name
  (p_application_id      IN number
  ,p_flex_code           IN varchar2
  ,p_flex_structure_code IN varchar2
  ,p_column_name         IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_id_flex_segments)
  $END
IS

  l_name fnd_id_flex_segments.segment_name%TYPE;
  
  CURSOR csr_name IS
  SELECT fifseg.segment_name
  FROM   fnd_id_flex_structures  fifs,
         fnd_id_flex_segments fifseg
  WHERE  fifs.application_id            = p_application_id
  AND    fifs.id_flex_code              = p_flex_code
  AND    fifs.id_flex_structure_code    = p_flex_structure_code
  AND    fifs.application_id            = fifseg.application_id
  AND    fifs.id_flex_code              = fifseg.id_flex_code
  AND    fifs.id_flex_num               = fifseg.id_flex_num
  AND    fifseg.application_column_name = p_column_name;
  
BEGIN

  IF p_application_id IS NULL OR
     p_flex_code      IS NULL OR
     p_column_name    IS NULL THEN
    RETURN NULL;
  ELSE
    OPEN  csr_name;
    FETCH csr_name INTO l_name;
    CLOSE csr_name;
  END IF;

  RETURN l_name;

END get_kff_segment_name;
  
--
-- ------------------------------------------------------------------------ *
-- Name    : get_dff_attribute_name
-- ------------------------------------------------------------------------ *
FUNCTION get_dff_attribute_name
  (p_application_id         IN number
  ,p_desc_flex_name         IN varchar2
  ,p_desc_flex_context_code IN varchar2
  ,p_column_name            IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_descr_flex_column_usages)
  $END
IS

  l_name fnd_descr_flex_column_usages.end_user_column_name%TYPE;
  
  CURSOR csr_name IS
  SELECT fdfcu.end_user_column_name segment_name
  FROM   fnd_descriptive_flexs_vl     fdf
  ,      fnd_descr_flex_contexts      fdfc
  ,      fnd_descr_flex_column_usages fdfcu
  WHERE  fdf.application_id                 = p_application_id
  AND    fdf.descriptive_flexfield_name     = p_desc_flex_name
  AND    fdf.application_id                 = fdfc.application_id
  AND    fdf.descriptive_flexfield_name     = fdfc.descriptive_flexfield_name
  AND   (fdfc.descriptive_flex_context_code = p_desc_flex_context_code OR
         fdfc.global_flag                   = 'Y')
  AND    fdfc.application_id                = fdfcu.application_id 
  AND    fdfc.descriptive_flexfield_name    = fdfcu.descriptive_flexfield_name
  AND    fdfc.descriptive_flex_context_code = fdfcu.descriptive_flex_context_code 
  AND    fdfcu.application_column_name      = p_column_name;
  
BEGIN

  IF p_application_id IS NULL OR
     p_desc_flex_name IS NULL OR
     p_column_name    IS NULL THEN
    RETURN NULL;
  ELSE
    OPEN  csr_name;
    FETCH csr_name INTO l_name;
    CLOSE csr_name;
  END IF;

  RETURN l_name;

END get_dff_attribute_name;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_kff_segment_prompt
-- ------------------------------------------------------------------------ *
FUNCTION get_kff_segment_prompt
  (p_application_id      IN number
  ,p_flex_code           IN varchar2
  ,p_flex_structure_code IN varchar2
  ,p_column_name         IN varchar2
  ,p_userenv_lang        IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_id_flex_segments_tl)
  $END
IS

  l_prompt fnd_id_flex_segments_vl.form_left_prompt%TYPE;
  
  CURSOR csr_prompt IS
  SELECT fifseg.form_left_prompt
  FROM   fnd_id_flex_structures  fifs,
         fnd_id_flex_segments_vl fifseg
  WHERE  fifs.application_id            = p_application_id
  AND    fifs.id_flex_code              = p_flex_code
  AND    fifs.id_flex_structure_code    = p_flex_structure_code
  AND    fifs.application_id            = fifseg.application_id
  AND    fifs.id_flex_code              = fifseg.id_flex_code
  AND    fifs.id_flex_num               = fifseg.id_flex_num
  AND    fifseg.application_column_name = p_column_name;
  
BEGIN

  IF p_application_id IS NULL OR
     p_flex_code      IS NULL OR
     p_column_name    IS NULL THEN
    RETURN NULL;
  ELSE
    OPEN  csr_prompt;
    FETCH csr_prompt INTO l_prompt;
    CLOSE csr_prompt;
  END IF;

  RETURN l_prompt;

END get_kff_segment_prompt;
  
--
-- ------------------------------------------------------------------------ *
-- Name    : get_dff_attribute_prompt
-- ------------------------------------------------------------------------ *
FUNCTION get_dff_attribute_prompt
  (p_application_id         IN number
  ,p_desc_flex_name         IN varchar2
  ,p_desc_flex_context_code IN varchar2
  ,p_column_name            IN varchar2
  ,p_userenv_lang           IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_descr_flex_col_usage_tl)
  $END
IS

  l_prompt fnd_descr_flex_col_usage_vl.form_left_prompt%TYPE;
  
  CURSOR csr_prompt IS
  SELECT fdfcu.form_left_prompt
  FROM   fnd_descriptive_flexs_vl     fdf
  ,      fnd_descr_flex_contexts      fdfc
  ,      fnd_descr_flex_col_usage_vl fdfcu
  WHERE  fdf.application_id                 = p_application_id
  AND    fdf.descriptive_flexfield_name     = p_desc_flex_name
  AND    fdf.application_id                 = fdfc.application_id
  AND    fdf.descriptive_flexfield_name     = fdfc.descriptive_flexfield_name
  AND   (fdfc.descriptive_flex_context_code = p_desc_flex_context_code OR
         fdfc.global_flag                   = 'Y')
  AND    fdfc.application_id                = fdfcu.application_id 
  AND    fdfc.descriptive_flexfield_name    = fdfcu.descriptive_flexfield_name
  AND    fdfc.descriptive_flex_context_code = fdfcu.descriptive_flex_context_code 
  AND    fdfcu.application_column_name      = p_column_name;
  
BEGIN

  IF p_application_id IS NULL OR
     p_desc_flex_name IS NULL OR
     p_column_name    IS NULL THEN
    RETURN NULL;
  ELSE
    OPEN  csr_prompt;
    FETCH csr_prompt INTO l_prompt;
    CLOSE csr_prompt;
  END IF;

  RETURN l_prompt;

END get_dff_attribute_prompt;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_dff_attributes
-- ------------------------------------------------------------------------ *
FUNCTION get_dff_attributes
  (p_application_id         IN number
  ,p_desc_flex_name         IN varchar2
  ,p_desc_flex_context_code IN varchar2) RETURN g_dff_attr_t
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_descr_flex_column_usages)
  $END
IS
  
  CURSOR csr_attr IS
  SELECT fdfcu.column_seq_num
        ,fdfcu.application_column_name
        ,fdfcu.end_user_column_name
        ,fdfcu.enabled_flag
        ,fdfcu.required_flag
        ,fdfcu.display_flag
        ,fdfcu.display_size
        ,fdfcu.maximum_description_len
        ,fdfcu.flex_value_set_id
        ,fvs.flex_value_set_name
        ,fdfcu.default_type
        ,fdfcu.default_value
        ,fdfcu.form_left_prompt
        ,fdfcu.form_above_prompt
        ,fdfcu.description
  FROM   fnd_descriptive_flexs_vl    fdf
  ,      fnd_descr_flex_contexts     fdfc
  ,      fnd_descr_flex_col_usage_vl fdfcu
  ,      fnd_flex_value_sets         fvs
  WHERE  fdf.application_id                 = 800
  AND    fdf.descriptive_flexfield_name     = 'PER_ASSIGNMENTS'
  AND    fdf.application_id                 = fdfc.application_id
  AND    fdf.descriptive_flexfield_name     = fdfc.descriptive_flexfield_name
  AND   (fdfc.descriptive_flex_context_code = null OR
         fdfc.global_flag                   = 'Y')
  AND    fdfc.application_id                = fdfcu.application_id 
  AND    fdfc.descriptive_flexfield_name    = fdfcu.descriptive_flexfield_name
  AND    fdfc.descriptive_flex_context_code = fdfcu.descriptive_flex_context_code
  AND    fdfcu.flex_value_set_id = fvs.flex_value_set_id(+)
  AND    fdfcu.enabled_flag = 'Y'
  ORDER BY fdfcu.column_seq_num;
  
  l_dff_attr_tbl g_dff_attr_t;
  i              number := 1;
  
BEGIN

  IF p_application_id IS NULL OR
     p_desc_flex_name IS NULL THEN
    RETURN l_dff_attr_tbl;
  ELSE
    FOR csr_rec IN csr_attr LOOP
      l_dff_attr_tbl(i).column_seq_num          := csr_rec.column_seq_num;
      l_dff_attr_tbl(i).application_column_name := csr_rec.application_column_name;
      l_dff_attr_tbl(i).end_user_column_name    := csr_rec.end_user_column_name;
      l_dff_attr_tbl(i).enabled_flag            := csr_rec.enabled_flag;
      l_dff_attr_tbl(i).required_flag           := csr_rec.required_flag;
      l_dff_attr_tbl(i).display_flag            := csr_rec.display_flag;
      l_dff_attr_tbl(i).display_size            := csr_rec.display_size;
      l_dff_attr_tbl(i).maximum_description_len := csr_rec.maximum_description_len;
      l_dff_attr_tbl(i).flex_value_set_id       := csr_rec.flex_value_set_id;
      l_dff_attr_tbl(i).flex_value_set_name     := csr_rec.flex_value_set_name;
      l_dff_attr_tbl(i).default_type            := csr_rec.default_type;
      l_dff_attr_tbl(i).default_value           := csr_rec.default_value;
      l_dff_attr_tbl(i).form_left_prompt        := csr_rec.form_left_prompt;
      l_dff_attr_tbl(i).form_above_prompt       := csr_rec.form_above_prompt;
      l_dff_attr_tbl(i).description             := csr_rec.description;
      i := i + 1;
    END LOOP;
  END IF;

  RETURN l_dff_attr_tbl;

END get_dff_attributes;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_xxas_msg_lud
-- ------------------------------------------------------------------------ *
FUNCTION get_xxas_msg_lud RETURN date
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_new_messages)
  $END
IS

  l_lud date;
  
BEGIN

  SELECT max(m.last_update_date)
  INTO   l_lud
  FROM   fnd_new_messages m
  WHERE  m.application_id =
    (SELECT xxas_util_pkg.app_id
     FROM   dual);

  RETURN l_lud;

END get_xxas_msg_lud;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_xxas_msg_lud_md5
-- ------------------------------------------------------------------------ *
FUNCTION get_xxas_msg_lud_md5 RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_new_messages)
  $END
IS

BEGIN

  RETURN xxas_util_pkg.md5(get_xxas_msg_lud);

END get_xxas_msg_lud_md5;

-- ------------------------------------------------------------------------ *
-- Name    : get_currency_symbol
-- ------------------------------------------------------------------------ *
FUNCTION get_currency_symbol
  (p_currency_code   IN varchar2) RETURN VARCHAR2
    $IF DBMS_DB_VERSION.VER_LE_10 $THEN    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_currencies)
  $END
IS
l_symbol varchar2(100);
BEGIN
  
  SELECT symbol
  INTO   l_symbol
  FROM  fnd_currencies where currency_code = p_currency_code;
  
  return l_symbol;
END get_currency_symbol;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_currency_precision
-- ------------------------------------------------------------------------ *
FUNCTION get_currency_precision
  (p_currency_code   IN VARCHAR2) RETURN number
    $IF DBMS_DB_VERSION.VER_LE_10 $THEN    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_currencies)
  $END
IS
l_precision number DEFAULT 2;
BEGIN
  
  SELECT precision
  INTO   l_precision
  FROM  fnd_currencies where currency_code = p_currency_code;

  return l_precision;
  
END get_currency_precision;

-- ------------------------------------------------------------------------ *
-- Name    : resolve_color_name
-- ------------------------------------------------------------------------ *
FUNCTION resolve_color_name
  (p_color IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_lookup_values)
  $END
IS

  l_PROC CONSTANT varchar2(30) := 'resolve_color_name';
  l_color         fnd_lookup_values.description%TYPE;
  
  CURSOR csr_resolve
  IS
  SELECT lower(flv.description)
  FROM   fnd_lookup_values flv
  WHERE  flv.lookup_type = 'XXAS_COM_COLOR_PALETTE'
  AND    flv.lookup_code = l_color;
  
BEGIN

  l_color := substr(upper(trim(p_color)), 1, 30);
  
  IF l_color = 'BALANCED' OR
     l_color = 'ENERGIZED' OR
     l_color = 'ASSERTIVE' OR
     l_color = 'POSITIVE'
  THEN
     l_color := lower(l_color);
  ELSE
    OPEN  csr_resolve;
    FETCH csr_resolve INTO l_color;
    CLOSE csr_resolve;
  END IF;
  
  RETURN lower(l_color);
  
END resolve_color_name;

-- ------------------------------------------------------------------------ *
-- Name    : hex_to_color_name
-- ------------------------------------------------------------------------ *
FUNCTION hex_to_color_name
  (p_color IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_lookup_values)
  $END
IS

  l_PROC CONSTANT varchar2(30) := 'hex_to_color_name';
  l_color         fnd_lookup_values.description%TYPE;
  
  CURSOR csr_color_name
  IS
  SELECT lower(flv.lookup_code)
  FROM   fnd_lookup_values flv
  WHERE  flv.lookup_type = 'XXAS_COM_COLOR_PALETTE'
  AND    lower(flv.description) = l_color;
  
BEGIN

  l_color := substr(lower(trim(p_color)), 1, 30);
  
  OPEN  csr_color_name;
  FETCH csr_color_name INTO l_color;
  CLOSE csr_color_name;
  
  RETURN nvl(l_color, p_color);
  
END hex_to_color_name;

-- ------------------------------------------------------------------------ *
-- Name    : get_message_string
-- ------------------------------------------------------------------------ *
FUNCTION get_message_string
  (p_message_name  IN VARCHAR2,
   p_language_code IN VARCHAR2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN

  $ELSE
    RESULT_CACHE RELIES_ON (fnd_new_messages)
  $END
IS

  CURSOR csr_msg
  IS
  SELECT fnm.message_text,
        fnm.message_number,
        fnm.type,
        fnm.fnd_log_severity,
        fnm.category,
        fnm.severity
    FROM fnd_new_messages fnm
   WHERE fnm.message_name  = p_message_name
    AND fnm.language_code  = p_language_code
    AND fnm.application_id = xxas_rslt_cache_pkg.g_APP_ID;

  l_message csr_msg%rowtype;

BEGIN

  IF p_message_name IS NOT NULL THEN
    OPEN  csr_msg;
    FETCH csr_msg INTO l_message;
    CLOSE csr_msg;
  END IF;

  RETURN l_message.message_text;

END get_message_string;

--
-- ------------------------------------------------------------------------ *
-- Name    : absences_func_id
-- ------------------------------------------------------------------------ *
FUNCTION absences_func_id RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN

  $ELSE
    RESULT_CACHE RELIES_ON (fnd_form_functions, fnd_form_functions_tl, fnd_menu_entries, fnd_menu_entries_tl, fnd_objects, xxas_object_values, fnd_menus, fnd_menus_tl)
  $END
IS

  CURSOR csr_function_id IS
  SELECT b.function_id
  FROM   xxas_block_flexible_v b
        ,xxas_pages_v p
  WHERE  b.menu_id        = p.menu_Id
  AND    b.obj_name       = 'XXAS_PER_ALL_ASSIGNMENTS_F'
  AND    b.form_handler IN ('xxas_leave_of_absence_pkg', 'xxas_absence_management_pkg')
  AND    p.published_flag = 'Y'
  AND    p.menu_name      <> 'XXAS_EMP_ABS_VIEW';

  l_function_id fnd_form_functions.function_id%type;

BEGIN

  OPEN  csr_function_id;
  FETCH csr_function_id INTO l_function_id;
  CLOSE csr_function_id;

  RETURN l_function_id;

END absences_func_id;

-- ------------------------------------------------------------------------ *
-- Name    : google_analytics_xml
-- ------------------------------------------------------------------------ *
FUNCTION google_analytics_xml RETURN varchar2 
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN

  $ELSE
    RESULT_CACHE RELIES_ON (fnd_profile_option_values)
  $END 
IS 
l_xmltype xmltype ; 
BEGIN
      SELECT XMLElement
                           ("googleAnalytics"
                           ,XMLAttributes
                             ('true' as "json:force-array"
                             ,'http://json.org/' as "xmlns:json")
                           ,XMLAgg
                             (XMLElement("Analytics",XMLConcat(XMLElement
                               ("trackingCode"
                               ,column_value)
                             ,XMLElement
                               ("autoLink"
                               ,( select xmlagg(XMLElement("Element",column_value))
                      from TABLE(xxas_util_pkg.string_to_varchar2tbl(fnd_profile.value('XXAS_COM_GOOGLE_ANALYTICS_LINKED_SITES'))))  
                             )
                           )))) 
       INTO l_xmltype
                   FROM   TABLE(xxas_util_pkg.string_to_varchar2tbl
                            (fnd_profile.value('XXAS_COM_GOOGLE_ANALYTICS_CODE')));
            RETURN xmltype.getStringVal(l_xmltype);     
            
END google_analytics_xml; 

-- ------------------------------------------------------------------------ *
-- Name    : get_message
-- ------------------------------------------------------------------------ *
FUNCTION get_message
  (p_message_app           IN varchar2
  ,p_message_name          IN varchar2
  ,p_language_code         IN varchar2
  ,p_message_token1_name   IN varchar2 DEFAULT NULL
  ,p_message_token1_value  IN varchar2 DEFAULT NULL
  ,p_message_token2_name   IN varchar2 DEFAULT NULL
  ,p_message_token2_value  IN varchar2 DEFAULT NULL
  ,p_message_token3_name   IN varchar2 DEFAULT NULL
  ,p_message_token3_value  IN varchar2 DEFAULT NULL
  ,p_message_token4_name   IN varchar2 DEFAULT NULL
  ,p_message_token4_value  IN varchar2 DEFAULT NULL
  ,p_message_token5_name   IN varchar2 DEFAULT NULL
  ,p_message_token5_value  IN varchar2 DEFAULT NULL
  ,p_message_token6_name   IN varchar2 DEFAULT NULL
  ,p_message_token6_value  IN varchar2 DEFAULT NULL
  ,p_message_token7_name   IN varchar2 DEFAULT NULL
  ,p_message_token7_value  IN varchar2 DEFAULT NULL
  ,p_message_token8_name   IN varchar2 DEFAULT NULL
  ,p_message_token8_value  IN varchar2 DEFAULT NULL
  ,p_message_token9_name   IN varchar2 DEFAULT NULL
  ,p_message_token9_value  IN varchar2 DEFAULT NULL
  ,p_message_token10_name  IN varchar2 DEFAULT NULL
  ,p_message_token10_value IN varchar2 DEFAULT NULL
  ,p_message_token11_name  IN varchar2 DEFAULT NULL
  ,p_message_token11_value IN varchar2 DEFAULT NULL
  ,p_message_token12_name  IN varchar2 DEFAULT NULL
  ,p_message_token12_value IN varchar2 DEFAULT NULL
  ,p_message_token13_name  IN varchar2 DEFAULT NULL
  ,p_message_token13_value IN varchar2 DEFAULT NULL
  ,p_message_token14_name  IN varchar2 DEFAULT NULL
  ,p_message_token14_value IN varchar2 DEFAULT NULL
  ,p_message_token15_name  IN varchar2 DEFAULT NULL
  ,p_message_token15_value IN varchar2 DEFAULT NULL) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN

  $ELSE
    RESULT_CACHE RELIES_ON (fnd_new_messages, fnd_application)
  $END
IS

BEGIN

  IF p_message_app   is not null AND 
     p_message_name  is not null AND 
     p_language_code is not null THEN
    RETURN xxas_com_msg_pkg.get_message(p_message_app           => p_message_app
                                       ,p_message_name          => p_message_name
                                       ,p_message_token1_name   => p_message_token1_name
                                       ,p_message_token1_value  => p_message_token1_value
                                       ,p_message_token2_name   => p_message_token2_name
                                       ,p_message_token2_value  => p_message_token2_value
                                       ,p_message_token3_name   => p_message_token3_name
                                       ,p_message_token3_value  => p_message_token3_value
                                       ,p_message_token4_name   => p_message_token4_name
                                       ,p_message_token4_value  => p_message_token4_value
                                       ,p_message_token5_name   => p_message_token5_name
                                       ,p_message_token5_value  => p_message_token5_value
                                       ,p_message_token6_name   => p_message_token6_name
                                       ,p_message_token6_value  => p_message_token6_value
                                       ,p_message_token7_name   => p_message_token7_name
                                       ,p_message_token7_value  => p_message_token7_value
                                       ,p_message_token8_name   => p_message_token8_name
                                       ,p_message_token8_value  => p_message_token8_value
                                       ,p_message_token9_name   => p_message_token9_name
                                       ,p_message_token9_value  => p_message_token9_value
                                       ,p_message_token10_name  => p_message_token10_name
                                       ,p_message_token10_value => p_message_token10_value
                                       ,p_message_token11_name  => p_message_token11_name
                                       ,p_message_token11_value => p_message_token11_value
                                       ,p_message_token12_name  => p_message_token12_name
                                       ,p_message_token12_value => p_message_token12_value
                                       ,p_message_token13_name  => p_message_token13_name
                                       ,p_message_token13_value => p_message_token13_value
                                       ,p_message_token14_name  => p_message_token14_name
                                       ,p_message_token14_value => p_message_token14_value
                                       ,p_message_token15_name  => p_message_token15_name
                                       ,p_message_token15_value => p_message_token15_value);
  END IF;

  RETURN null;

END get_message;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_appl_list_func_id
-- ------------------------------------------------------------------------ *
FUNCTION get_appl_list_func_id
    (p_slug IN VARCHAR2) RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_menu_entries)
  $END
IS

  CURSOR csr_function_id 
  IS
  SELECT fm.function_id 
  FROM  fnd_menu_entries fm,
        xxas_pages_v p
  WHERE p.slug = p_slug
  AND   p.published_flag = 'Y'
  AND   p.menu_id = fm.menu_id;
  
  l_function_id fnd_form_functions.function_id%type;

BEGIN

  OPEN  csr_function_id;
  FETCH csr_function_id INTO l_function_id;
  CLOSE csr_function_id;

  RETURN l_function_id;
  
END get_appl_list_func_id;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_offer_details_block_id
-- ------------------------------------------------------------------------ *
FUNCTION get_offer_details_block_id
    (p_slug IN VARCHAR2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_menu_entries)
  $END
IS
 
  l_return varchar2(400);

BEGIN

  SELECT (to_char(b.function_id) ||  '-activeBlockId=' || to_char(fm.function_id)) into l_return
  FROM xxas_block_blocks_v b,
       fnd_menu_entries_vl fm,
       xxas_pages_v p
  WHERE p.slug = p_slug
  AND   p.published_flag = 'Y'
  AND   p.menu_id = b.menu_id
  AND   xxas_rslt_cache_pkg.menu_name_to_id(b.child_menu) = fm.menu_id
  AND   lower(fm.prompt) like '%offer%';

  RETURN l_return;
  
END get_offer_details_block_id;

--
-- ------------------------------------------------------------------------ *
-- Name    : plan_review_chart_rc
-- ------------------------------------------------------------------------ *
FUNCTION plan_review_chart_rc
  (p_keys IN xxas_com_keys_type) RETURN XMLType
IS

  l_PROC CONSTANT varchar2(80) := 'plan_review_chart_rc';
  l_xml           XMLType;

BEGIN

  SELECT XMLElement
           ("chart"
           ,XMLElement
             ("type", 'PieChart')
           ,XMLElement
             ("data"
             ,XMLElement
               ("cols"
               ,XMLAttributes
                  ('true' as "json:force-array"
                  ,'http://json.org/' as "xmlns:json")
               ,XMLElement
                 ("column"
                 ,XMLForest
                   ('Status' "label" /* TODO MLS */
                   ,'string' "type")
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   ('Number of reviews' "label" /* TODO MLS */
                   ,'number' "type")
                 )
               ,XMLElement
                 ("column"
                 ,XMLForest
                   ('tooltip' "role"
                   ,'string' "type")
                 )
               )
              ,XMLElement
                ("rows"
                ,XMLAttributes
                   ('true' as "json:force-array"
                   ,'http://json.org/' as "xmlns:json")
                ,XMLAgg
                  (XMLElement
                    ("row"
                    ,XMLElement
                      ("c"
                       ,XMLAttributes
                        ('true' as "json:force-array"
                        ,'http://json.org/' as "xmlns:json")
                      ,XMLElement
                        ("value"
                        ,XMLElement
                          ("v",  status_meaning)
                        )
                      ,XMLElement
                        ("value"
                        ,XMLElement
                           ("v",  enrollment_count)
                        )
                      ,XMLElement
                        ("value"
                        ,XMLElement
                          ("v",  names)
                        )
                      )
                    )  ORDER BY status_meaning
                  )
                )
              )
           ,XMLElement
              ( "options"
              ,XMLElement("legend"
                         ,XMLElement("position"
                                    ,'bottom'
                                    )
                        /*,XMLElement("textStyle"
                                   ,XMLForest('#666666' "color"
                                             ,10 "fontSize")
                                    )*/
                         )
              ,XMLElement("pieHole"
                         ,.5)             
             /* ,XMLElement("pieSliceTextStyle"
                         ,XMLElement("fontSize"
                                    ,14))*/
              ,XMLElement
                 ("colors"
                 ,XMLAttributes
                    ('true' as "json:force-array"
                    ,'http://json.org/' as "xmlns:json")
                 ,XMLAgg
                   (XMLElement
                     ("color"
                     ,xxas_rslt_cache_pkg.resolve_color_name(status_color)
                     ) ORDER BY status_meaning
                   )
                 )
              ,XMLElement
                 ("animation"
                 ,XMLElement("startup" , 'true')
                 ,XMLElement("duration" , 1000)
                 )            
              ,XMLElement
                 ("tooltip"
                 ,XMLElement("isHtml" , 'false')
                 ,XMLElement
                   ("textStyle"
                   ,XMLElement("showColorCode", 'true')
                   )
                 )
               )
             ) "data"
  INTO  l_xml
  FROM  (SELECT  xprf.status_meaning
                ,count(*) enrollment_count
                ,CASE xprf.status_code
                   WHEN 'NS' THEN 'red'
                   WHEN 'D'  THEN 'orange'
                   WHEN 'A'  THEN 'green'
                   WHEN 'O'  THEN 'green'
                   WHEN 'X'  THEN 'grey'
                   ELSE 'grey'
                 END status_color
                 $IF DBMS_DB_VERSION.VER_LE_11 OR DBMS_DB_VERSION.VER_LE_10 $THEN

                   ,rtrim(xmlagg(xmlelement(k, xxas_util_pkg.get_formatted_name
                        (papf.full_name
                        ,papf.global_name
                        ,papf.local_name
                        ,papf.first_name
                        ,papf.last_name
                        ,papf.known_as), ', ') order by null).extract('//text()'), ', ' ) 
                $ELSE
                    ,rtrim(listagg(xxas_util_pkg.get_formatted_name
                     (papf.full_name
                     ,papf.global_name
                     ,papf.local_name
                     ,papf.first_name
                     ,papf.last_name
                     ,papf.known_as), ', ' ON OVERFLOW TRUNCATE '...') within group (order by null) , ', ' ) 
                $END  names
          FROM   xxas_perf_plan_reviews xppr
                ,xxas_perf_plan_enrollments xppe
                ,xxas_perf_review_forms_v xprf
                ,per_all_people_f papf
                ,per_all_assignments_f paaf
          WHERE  xppr.perf_plan_review_id = p_keys.get_key('PERF_PLAN_REVIEW_ID')
          AND    xppe.perf_plan_id = xppr.perf_plan_id
          AND    xprf.perf_plan_enrollment_id = xppe.perf_plan_enrollment_id
          AND    xprf.perf_review_template_id = xppr.perf_review_template_id
          AND    xppe.person_id = papf.person_id
          AND    paaf.person_id = xppe.person_id
          AND    paaf.assignment_id = xppe.assignment_id
          AND    paaf.primary_flag = 'Y'
          AND    trunc(sysdate) BETWEEN
                 paaf.effective_start_date AND paaf.effective_end_date
          AND    trunc(sysdate) BETWEEN
                 papf.effective_start_date AND papf.effective_end_date
          AND    papf.person_id IN
                   (SELECT column_value
                    FROM   TABLE(xxas_com_hier_pkg.drs_for_per_tbl
                                  (p_keys.get_key('PERSON_ID'), trunc(sysdate))))
          GROUP BY xprf.status_meaning
                  ,CASE xprf.status_code
                     WHEN 'NS' THEN 'red'
                     WHEN 'D'  THEN 'orange'
                     WHEN 'A'  THEN 'green'
                     WHEN 'O'  THEN 'green'
                     WHEN 'X'  THEN 'grey'
                     ELSE 'grey'
                   END
        );

  RETURN l_xml;

END plan_review_chart_rc;

END xxas_rslt_cache_pkg;
/
COMMIT;
EXIT;
